/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */


//ensure CJSE & QT SFW namespace are created.
if (window.wrt === undefined) {
   /**
    * @namespace
    */
    wrt = {};
    wrt.device = {};
    wrt._services = {};
}
else {
    if (window.wrt._services === undefined) {
        wrt._services = {};
    }
    if (window.wrt.device === undefined){
        wrt.device = {};
    }
}

wrt._services = (function(){
    var error = null;
    /*
     * Implemention of wrt._services.load method
     *
     * @function {public object} _sfw_service_load
     * @param {string} serviceName service name of requested service
     * @param {string} interfaceName interface name of requested service
     * @param {string} version version of requested service
     * @returns service object
     */
    function _sfw_service_load(serviceName, interfaceName, version){
        //ensure qt wrt._services is been intialised.
        var DeviceException = wrt.device.DeviceException;
        if (!error)
            error = new DeviceException('dummy',0);
        createFactoryObject();

        try {
            var so = wrt.factory.getServiceObject(serviceName, interfaceName);
            return so;
        }
        catch (e) {
            sfw_debug(serviceName + " " + interfaceName + " not available:"+e);
            throw new DeviceException(error.DATA_NOT_FOUND_ERR,"failed to load requested service interface, enabler not found or access restricted");
        }
    }

    /*
     * Implemention of the wrt._services.listServices method
     *
     * @function {public array} _sfw_service_list
     * @returns list of registered services
     */
    function _sfw_service_list()
    {
        try {
            createFactoryObject();
            var l = wrt.factory.getServiceProviders();
            return l;
        }
        catch (e){
            sfw_debug("wrt.factory.getServiceProviders: "+e);
            return null;
        }
    }

    /*
     *
     */
    function _sfw_service_init(callback) {
        try {
            createFactoryObject();

            // Automatically callback to client after 1 MS
            setTimeout(callback, 1 );
        }
        catch(ex) {
            sfw_debug(ex);
        }
    }

    /*
     * @var {bool} sfw_debug_on is ture if in debug mode
     */
    var sfw_debug_on = false;

    /*
     * @function {public void} sfw_debug if sfw_debug_on = true alert the input message
     * @param {string} message
     */
    function sfw_debug(message)
    {
        if (sfw_debug_on) {
            alert(message);
        }
    }

    /*
     * Sleep for milliseconds
     *
     * @function {public void} pausecomp
     */
    function pausecomp(millis) {
        var date = new Date();
        var curDate = null;
        do {
            curDate = new Date();
        } while (curDate-date < millis);
    }

// Fix to improve performance in Converting QVariants to NP Objects.

function __newObject(){
   return new Object();
};

function __newArray(){
   return new Array();
};

function __newDate(){
   return new Date();
};

    /*
     * Creates a factory object and embeds it to html file
     *
     * @function {public object} createFactoryObject
     * @returns factory plugin
     */
    function createFactoryObject() {
        var factory = document.getElementById("serviceFactory");
        //plugin is not embeded yet
        if (!factory) {
            //embed plugin in a div node to html file
            var newdiv = document.createElement('div');
            newdiv.id = "factoryDiv";
            newdiv.innerHTML = '<embed  id="serviceFactory" type="application/x-sfw-plugin" height="0" hidden=true > </embed>';
            if (document.body)
               document.body.insertBefore(newdiv, document.body.firstChild);
            else
               document.getElementsByTagName('html').item(0).appendChild( newdiv );

            // Setup the wrt.factory
            wrt.factory = document.getElementById("serviceFactory");
        // Call this again to execute else case
        setTimeout(createFactoryObject, 1 );
        }
        else {
            // If the client already has the plugin embedded
            wrt.factory=factory;
            wrt.factory.SFW_CreateJSObject(__newObject,__newArray,__newDate);
        }
    }
    return{
        load : _sfw_service_load,
        listServices: _sfw_service_list,
        init:_sfw_service_init,
        version: "0.1",
        info: "Service Framework",
        factory: null
    };
})();


/**
 * The wrt.device object allows widgets to access platform services through Platform Services 2.0 APIs. The wrt.device 
 * object is used to create service objects for specific service interfaces. These service objects can then be used to 
 * access the methods defined in the interfaces. The service objects and methods are described in the Platform Services 2.0 
 * API reference. </br> </br>
 * The wrt.device object can be accessed simply via wrt.device. You do not need to create an object instance for it. </br> </br>
 * Note: In Symbian Web Runtime versions 1.1 and later, the wrt.device object is not a built-in object in the Web Runtime environment. 
 * It is supported only through Platform Services 2.0. To use the wrt.device object and thus the Platform Services 2.0 APIs in a widget, 
 * you must import the Platform Services 2.0 JavaScript library into the widget. </br> </br>
 * Note: For Qt WRT 1.1 and later, the Device API (Platform Services 2.0) JavaScript library is automatically loaded into the main frame 
 * of the Web App (widget). 
 *
 * @namespace
 */
wrt.device = (function(){

    var cjse_interfaces = [];
    var xhr = null;
    var error = null;

    function getSharedLibUri(interfaceName){
        var SHARED_LIB='lib://';
        // In jscoverage mode SHARED_LIB_PATH is defined in testframework ctesttree.js
        if((window._$jscoverage || window.parent._$jscoverage) && window.SHARED_LIB_PATH){
            SHARED_LIB = window.SHARED_LIB_PATH;
        }
        var CJSE_NS ='wrt/device/';
        var JS_EXTN ='.js';

        return  (SHARED_LIB+CJSE_NS+interfaceName+JS_EXTN);
    }

    function getScriptSrc(/*URI*/uri, /*Boolean*/ fail_ok){
        var DeviceException = wrt.device.DeviceException;
        var msg=null;

        if (!error)
            error = new DeviceException('dummy',0);

        try{
            if (!xhr)
                xhr = new XMLHttpRequest();
        }
        catch(e){
            throw new DeviceException(error.NOT_SUPPORTED_ERR,'XHr not supported');
        }

        try {
            xhr.open('GET', uri, false);
            xhr.send(null);
            if (xhr.status){
                msg = "XHR: Unable to GET requested inteface reason not found: " + uri + " status:" + xhr.status + xhr.responseText;
                throw new DeviceException(error.DATA_NOT_FOUND_ERR,msg);
            }
        }
        catch (e) {
            if (fail_ok) {
                return null;
            } // null
            if (e.name != 'DeviceException'){
                msg = "XHR: Unable to GET requested inteface reason not found : "+ e.message;
                throw new DeviceException(error.DATA_NOT_FOUND_ERR,msg);
            }
        }
        return xhr.responseText;
    }

    function _ps_list_interfaces(){
        function __FWIter(arr)  {
            this._arr = arr;
            this._pos = 0;
            this._valid = true;

            this.hasNext = function() {
                return (this._valid && this._pos < this._arr.length)
            };

            this.next = function() {
                if (this._valid && (this._pos < this._arr.length)){
                    return this._arr[this._pos++];
                }
                else{
                    return null;
                }
            };

            this.close = function(){
                delete this._arr;
                this._valid = false;
            }
        }

        if (!cjse_interfaces.length || !wrt.device._interfaces){
            var services = wrt._services.listServices();
            wrt.device._interfaces = {};
            for (i=0;i<services.length;i++)
            {
                if (services[i].toLowerCase().lastIndexOf('wrt.device.') != -1){
                    var cjse_interface = services[i].split('wrt.device.')[1];

                    /* TODO Hardcoded version needs to be removed. No service version 
                    information is available from qtmobility. */
                    var version = wrt.device._interfaceVersion;

                    cjse_interfaces.push({name:cjse_interface,version:version});
                }
            }
        }

        return new __FWIter(cjse_interfaces);
    }

    return{
        /**
        * @memberOf wrt.device
        * @description interfaces internal
        */
        _interfaces: null,

        /**
        * @function
        * @description The load method creates an object instance of a service interface: a service 
        * object. The service object is used to access a platform service. For example, the Platform 
        * Services 2.0 Calendar API provides the calendar service object for accessing the Calendar Service.
        * The load method also associates the service object with namespace wrt.device.<InterfaceName>
        *
        * @param {String}  interfaceName  Specifies the fully qualified name of the service interface 
        * (service object) to load. For a list of available service interface names, see the following table.
        * @param {Number}  [version=null]  Specifies the minimum service interface version to load. For example, 1.0.
        * If the version supported by the device is the same or newer than the specified version, the 
        * service object is created normally. If the supported version is older than the specified version, 
        * the service object is not created and the method returns null.
        * If you do not specify a version, the latest supported version is used to create the service object.
        *
        * @throws {MISSING_ARG_ERR}  A mandatory argument in a method is missing.
        * @throws {INVALID_ARG_ERR}  The value of a method parameter is not of the expected type.
        * @throws {DATA_NOT_FOUND_ERR}  The requested interface or device enabler is unable to load because it was not found.
        *
        * @return {wrt.device.ServiceInterface}   Method returns requested ServiceInterface object. This is the requested service object.
        * It also associates the service object with namespace wrt.device.contacts i.e {@link wrt.device.contacts}. For example, after 
        * the following method call, contacts and wrt.device.contacts refer to the same service object instance: </br> </br>
        * <code> var contacts = nokia.device.load("contacts"); </code>.
        *
        * @example 
        * // code snippet for loading the contacts service 
        * // ---------------------------------------------------
        * // global contacts interface object
        * var so;
        * 
        * try {
        * // Loading the interface can be done in two ways
        * 
        * // Option 1 (Recommended): Use listInterfaces() to retrieve the available interfaces and
        * // use the returned iterator to provide the interface name and version
        * var interfaces = wrt.device.listInterfaces();
        * var loadInterface = null;
        * while(loadInterface = interfaces.next()) {
        *     if (loadInterface.name == "contacts")
        *         break;
        * }
        * if (loadInterface)
        *     so = wrt.device.load(loadInterface.name, loadInterface.version);
        * else
        *     alert("Contacts interface not found");
        * 
        * // Option 2: Provide the interface name and version directly
        * so = wrt.device.load("contacts", 2.0);
        * 
        * // You can now use 'so' to call the methods defined in the contacts interface:
        * // so.addContact();
        * // ...
        * 
        * // You can also use 'so' to retrieve interface name and version information:
        * // so.interfaceName;
        * // so.version;
        * }
        * catch(e) {
        *     alert(e);
        * }
        */
        load: function(interfaceName, version){
            var DeviceException = wrt.device.DeviceException;
            var error = new DeviceException('dummy',0);

            if (interfaceName){
                if (typeof interfaceName != 'string'){
                throw new DeviceException(error.INVALID_ARG_ERR, 'wrt.device.load:Invalid type interfaceName');
                }
            }else{
                throw new DeviceException(error.MISSING_ARG_ERR, 'wrt.device.load:interfaceName param expected');
            }

            if (version){
                if (typeof version != 'number'){
                    throw new DeviceException(error.INVALID_ARG_ERR, 'wrt.device.load:Invalid type version');
                }
            }

            if ( !(wrt.device[interfaceName]) || interfaceName == 'audioplayer' ) {
                var scriptUri = getSharedLibUri(interfaceName);
                var scriptSrc = getScriptSrc(scriptUri);
                window.eval(scriptSrc);
            }
            return wrt.device[interfaceName];
        },
        /**
         * @function
         * @description The listInterfaces method retrieves a list of available service interfaces.
         * @return {wrt.device.Iterator} of {@link wrt.device.InterfaceInfo} object's.
         * @example 
         * // code snippet for retrieving a list of available interfaces
         * // ----------------------------------------------------------
         * var requestInterface = "contacts";
         * try {
         * // retrieve a list of available interfaces
         * 
         * var availableInterfaces = nokia.device.listInterfaces();
         *
         * // check if requestInterface is among the available interfaces
         * var interface;
         * if (typeof availableInterfaces == "object") {
         *      while(interface = availableInterfaces.next()) {
         *          if (interface.name == requestInterface)
         *              alert("Operation successful: " + requestInterface + " found");
         * // To ensure fail-safe execution, check whether the API service interface version 
         * // is greater than or equal to the Platform Services 2.0 JS library version. 
         *              if{interface.version < nokia.device.getLibraryVersion())  {
         *                  alert("Requested API service interface version is less than the PS 2.0 library version. Using this individual API version might cause problems during execution.");
         *              }
         *          } 
         *      }
         * }
         * catch(e) {
         *      alert(e);
         * }
         */
        listInterfaces: _ps_list_interfaces,
        /**
         * @function
         * @description Method retrieves information about the value limitations of system-wide properties whose 
         * allowed values can vary across device platforms. When you call this method, the returned information 
         * describes the value limitations on the current platform.
         * @return {wrt.device.SystemProperties}  Which describes the property-specific value limitations on the current platform.
         * @example
         * // code snippet for identifying system-wide value limitations
         * // ----------------------------------------------------------
         * 
         *  try {
         *      var systemProperties = nokia.device.getSystemProperties();
         *      alert("Max URI length: " + systemProperties.uriPath.max);
         *  }
         *  catch(e) {
         *      alert(e);
         *  }
         */
        getSystemProperties: function (){
        return {
                uriPath:{
                    // maximum limitation is 255 charectors
                    max:255
                }
            };
        },
        /**
         * @function
         * @description Method retrieves the version of the Platform Services JavaScript library that is in the device 
         * for use by the widget.
         * @return {Number}  Returns the version of the Platform Services JavaScript library used by the widget. For example, 2.0, 2.2 3.0.
         * @example
         * // code snippet for retrieving the Platform Services JavaScript library version
         * // -----------------------------------------------------------------
         *
         * try {
         *      var libraryVersion = nokia.device.getLibraryVersion();
         *      alert("Platform Services JavaScript library version: " + libraryVersion);
         *  }
         *  catch(e) {
         *      alert(e);
         *  }
         */
        getLibraryVersion: function (){
            return 3.0;
        }
    };
})();

/*
* This _interfaceVersion property is used by each module in
* construction to initialize module's service object version 
* property.
*/
wrt.device._interfaceVersion = 3.0;

//wrt.device.__defineSetter__("version",function(){throw new Error("version readonly property cannot be set");});

/**
 * @class
 * @description Object stores the exception information for synchronous and asynchronous methods. This object 
 * is returned in the catch block of the try-catch statement for the method. You can extract the exception 
 * information from the properties of the DeviceException object. DeviceException extends Error by adding the code property.
 *
 * @param {Number} code A code number for the cause of the exception.
 * @param {String} message A free-form message explaining why this exception is thrown.
 */
wrt.device.DeviceException = function(code,message)    {

        /**
        * Contains the name of the exception. This is always "DeviceException".
        * @type String
        */
        this.name    = 'DeviceException';
        /**
        * A message contains a human-readable description of the exception or error.
        * @type String
        */
        this.message = message;
        /**
         * Exception cause as a code number. 
         * The error code number range is divided as follows:
         * <ul>
         * <dd> 1-49 are reserved for generic non-recoverable errors. </dd>
         * <dd> 50-99 are reserved for domain-specific non-recoverable errors. An error code in one domain can have 
         * a different meaning in another domain. </dd>
         * <dd> 100-499 are reserved for generic recoverable errors. </dd>
         * <dd> 500 and above are reserved for domain-specific recoverable errors. An error code in one domain can 
         * have a different meaning in another domain. </dd>
         * </ul>
         * @type Number
         */
        this.code    = code;
        /**
        * A mandatory parameter is missing from the method call.
        * @type Number
        * @constant
        */
        this.MISSING_ARG_ERR            = 1;
        /**
        * The value of a method parameter is not of the expected type.
        * @type Number
        * @constant
        */
        this.INVALID_ARG_ERR            = 2;
        /**
        * A data schema element is not supported by the current platform.
        * The specified method is not available in the API.
        * @type Number
        * @constant
        */
        this.NOT_SUPPORTED_ERR          = 3;
        /**
        * The system could not provide a response within a given time.
        * @type Number
        * @constant
        */
        this.TIMEOUT_ERR                = 100;
        /**
        * The specified data entry was not found.
        * @type Number
        * @constant
        */
        this.DATA_NOT_FOUND_ERR         = 101;
        /**
        * The specified data entry already exists.
        * @type Number
        * @constant
        */
        this.DATA_ALREADY_EXISTS_ERR    = 102;
        /**
        * The service is busy and unable to accept requests at the moment. Try again later.
        * @type Number
        * @constant
        */
        this.SERVICE_BUSY_ERR           = 103;
        /**
        * You have already registered to receive notifications. You can register only once per service.
        * @type Number
        * @constant
        */
        this.SERVICE_IN_USE_ERR         = 104;
        /**
        * The specified value is out of the expected range.
        * @type Number
        * @constant
        */
        this.DATA_OUT_OF_RANGE_ERR      = 105;
        /**
        * You do not have permission to access the specified content.
        * @type Number
        * @constant
        */
        this.NOT_ALLOWED_ERR            = 106;
        /**
        * Data size exceeded the allowed size limit set by the platform or the configuration. 
        * This is typically a data overflow error.
        * @type Number
        * @constant
        */
        this.SIZE_EXCEEDED_ERR          = 107;
        /**
        * The specified URI is invalid. It does not conform to the URI schema.
        * @type Number
        * @constant
        */
        this.INVALID_URI_ERR            = 108;
        /**
        *The specified URI refers to a non-existing resource.
        * @type Number
        * @constant
        */
        this.URI_NOT_FOUND_ERR          = 109;
        /**
        * The specified URI refers to an existing resource and the resource cannot be overwritten.
        * @type Number
        * @constant
        */
        this.URI_ALREADY_EXISTS_ERR     = 110;
        /**
        * The device ran out of memory while processing the request.
        * @type Number
        * @constant
        */
        this.NO_MEMORY_ERR              = 111;
        /**
        * The specified directory is not empty. Specific to {@link wrt.device.filesystem} API.
        * @type Number
        * @constant
        */
        this.DIR_NOT_EMPTY_ERR          = 500;
        /**
        * File is in use Specific to {@link wrt.device.filesystem} API.
        * @type Number
        * @constant
        */
        this.FILE_ALREADY_OPEN_ERR      = 501;
        /**
        * The specified URI is not supported. Specific to {@link wrt.device.filesystem} API..
        * @type Number
        * @constant
        */
        this.URI_NOT_SUPPORTED          = 502;
        /**
        * An unidentified error occurred.
        * @type Number
        * @constant
        */
        this.CONNMGR_UNKNOWN_SESSION_ERROR      = 501;
        /**
        * The session was aborted by the user or system.
        * @type Number
        * @constant
        */
        this.CONNMGR_SESSION_ABORTED_ERROR      = 502;
        /**
        * The session cannot roam to the new configuration.
        * @type Number
        * @constant
        */
        this.CONNMGR_ROAMING_ERROR              = 503;
        /**
        * The operation is not supported for current configuration.
        * @type Number
        * @constant
        */
        this.CONNMGR_OPERATION_NOTSUPPORTED_ERR = 504;
        /**
        * The operation cannot currently be performed for the current configuration. 
        * User can try this operation after some time.
        * @type Number
        * @constant
        */
        this.CONNMGR_NO_CONFIGURATION_ERROR     = 505;
        /**
        * Connection Refused
        * @type Number
        * @constant
        */
        this.FU_CONNECTION_REFUSED_ERR        = 601;
        /**
        * Remote Host Closed
        * @type Number
        * @constant
        */
        this.FU_REMOTE_HOST_CLOSED_ERR        = 602;
        /**
        * Host Not Found
        * @type Number
        * @constant
        */
        this.FU_HOST_NOT_FOUND_ERR          = 603;
        /**
        * Time Out
        * @type Number
        * @constant
        */
        this.FU_TIME_OUT_ERR            = 604;
        /**
        * Operation Canceled
        * @type Number
        * @constant
        */
        this.FU_OPERATION_CANCELLED_ERR       = 605;
        /**
        * SSL Handhake Failed
        * @type Number
        * @constant
        */
        this.FU_SSL_HANDSHAKE_FAILED        = 606;
        /**
        * Unknown Network Error
        * @type Number
        * @constant
        */
        this.FU_UNKNOWN_NETWORK_ERR         = 699;
        /**
        * Proxy Connection Refused
        * @type Number
        * @constant
        */
        this.FU_PROXY_CONECTION_REFUSED_ERR     = 701;
        /**
        * Proxy Connection Closed
        * @type Number
        * @constant
        */
        this.FU_PROXY_CONNECTION_CLOSED_ERR     = 702;
        /**
        * Proxy Not Found
        * @type Number
        * @constant
        */
        this.FU_PROXY_NOT_FOUND_ERR         = 703;
        /**
        * Proxy time Out
        * @type Number
        * @constant
        */
        this.FU_PROXY_TIME_OUT_ERR          = 704;
        /**
        * Proxy Authentication Required
        * @type Number
        * @constant
        */
        this.FU_PROXY_AUTHENTICATION_REQUIRED_ERR   = 705;
        /**
        * Unknown Proxy Error
        * @type Number
        * @constant
        */
        this.FU_UNKNOWN_PROXY_ERR           = 799;
        /**
        * Content Access Denied
        * @type Number
        * @constant
        */
        this.FU_CONTENT_ACCESS_DENIED_ERR       = 801;
        /**
        * Content Operation Not Permitted
        * @type Number
        * @constant
        */
        this.FU_CONTENT_NOT_PERMITTED_ERR       = 802;
        /**
        * Content Not Found
        * @type Number
        * @constant
        */
        this.FU_CONTENT_NOT_FOUND_ERR         = 803;
        /**
        * Authentication Required
        * @type Number
        * @constant
        */
        this.FU_AUTHENTICATION_REQUIRED_ERR     = 804;
        /**
        * Content Resend
        * @type Number
        * @constant
        */
        this.FU_CONTENT_RESEND_ERR          = 805;
        /**
        * Unknown Content Error
        * @type Number
        * @constant
        */
        this.FU_CONTENT_UNKNOWN_ERR         = 899;
        /**
        * Protocol Unknown Error
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_UNKNOWN_ERR        = 901;
        /**
        * Protocol Invalid Operation
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_INVALID_OPERATION      = 902;
        /**
        * Protocol Failure
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_FAILURE_ERR        = 999;
    }

wrt.device.DeviceException.prototype = new Error();

wrt.device.DeviceException.prototype.constructor = wrt.device.DeviceException;

wrt.device.DeviceException.prototype.toString = function(){
    return(this.name + " : " + this.code + " : " + this.message);
};


/**
 * @class
 * @description Object stores the error information for an asynchronous methods. This object is 
 * returned in errorCallback function of asynchronous methods.
 *
 * @param {Number} code A code number for the cause of the error.
 * @param {String} message A free-form message explaining why this error occured.
 * @return wrt.device.DeviceAPIError
 */
wrt.device.DeviceAPIError = function (code,message)    {
        /**
        * Contains the name of the error. This is always "DeviceAPIError".
        * @type String
        */
        this.name    = 'DeviceAPIError';
        /**
        * A message contains a human-readable description of the error.
        * @type String
        */
        this.message = message;
        /**
         * Error cause as a code number. 
         * The error code number range is divided as follows:
         * <ul>
         * <dd> 1-49 are reserved for generic non-recoverable errors. </dd>
         * <dd> 50-99 are reserved for domain-specific non-recoverable errors. An error code in one domain can have 
         * a different meaning in another domain. </dd>
         * <dd> 100-499 are reserved for generic recoverable errors. </dd>
         * <dd> 500 and above are reserved for domain-specific recoverable errors. An error code in one domain can 
         * have a different meaning in another domain. </dd>
         * </ul>
         * @type Number
         */
        this.code    = code;
        /**
        * A mandatory parameter is missing from the method call.
        * @type Number
        * @constant
        */
        this.MISSING_ARG_ERR            = 1;
        /**
        * The value of a method parameter is not of the expected type.
        * @type Number
        * @constant
        */
        this.INVALID_ARG_ERR            = 2;
        /**
        * A data schema element is not supported by the current platform.
        * The specified method is not available in the API.
        * @type Number
        * @constant
        */
        this.NOT_SUPPORTED_ERR          = 3;
        /**
        * The system could not provide a response within a given time.
        * @type Number
        * @constant
        */
        this.TIMEOUT_ERR                = 100;
        /**
        * The specified data entry was not found.
        * @type Number
        * @constant
        */
        this.DATA_NOT_FOUND_ERR         = 101;
        /**
        * The specified data entry already exists.
        * @type Number
        * @constant
        */
        this.DATA_ALREADY_EXISTS_ERR    = 102;
        /**
        * The service is busy and unable to accept requests at the moment. Try again later.
        * @type Number
        * @constant
        */
        this.SERVICE_BUSY_ERR           = 103;
        /**
        * You have already registered to receive notifications. You can register only once per service.
        * @type Number
        * @constant
        */
        this.SERVICE_IN_USE_ERR         = 104;
        /**
        * The specified value is out of the expected range.
        * @type Number
        * @constant
        */
        this.DATA_OUT_OF_RANGE_ERR      = 105;
        /**
        * You do not have permission to access the specified content.
        * @type Number
        * @constant
        */
        this.NOT_ALLOWED_ERR            = 106;
        /**
        * Data size exceeded the allowed size limit set by the platform or the configuration. 
        * This is typically a data overflow error.
        * @type Number
        * @constant
        */
        this.SIZE_EXCEEDED_ERR          = 107;
        /**
        * The specified URI is invalid. It does not conform to the URI schema.
        * @type Number
        * @constant
        */
        this.INVALID_URI_ERR            = 108;
        /**
        *The specified URI refers to a non-existing resource.
        * @type Number
        * @constant
        */
        this.URI_NOT_FOUND_ERR          = 109;
        /**
        * The specified URI refers to an existing resource and the resource cannot be overwritten.
        * @type Number
        * @constant
        */
        this.URI_ALREADY_EXISTS_ERR     = 110;
        /**
        * The device ran out of memory while processing the request.
        * @type Number
        * @constant
        */
        this.NO_MEMORY_ERR              = 111;
        /**
        * The specified directory is not empty. Specific to {@link wrt.device.filesystem} API.
        * @type Number
        * @constant
        */
        this.DIR_NOT_EMPTY_ERR          = 500;
        /**
        * File is in use Specific to {@link wrt.device.filesystem} API.
        * @type Number
        * @constant
        */
        this.FILE_ALREADY_OPEN_ERR      = 501;
        /**
        * The specified URI is not supported. Specific to {@link wrt.device.filesystem} API..
        * @type Number
        * @constant
        */
        this.URI_NOT_SUPPORTED          = 502;
        /**
        * An unidentified error occurred.
        * @type Number
        * @constant
        */
        this.CONNMGR_UNKNOWN_SESSION_ERROR      = 501;
        /**
        * The session was aborted by the user or system.
        * @type Number
        * @constant
        */
        this.CONNMGR_SESSION_ABORTED_ERROR      = 502;
        /**
        * The session cannot roam to the new configuration.
        * @type Number
        * @constant
        */
        this.CONNMGR_ROAMING_ERROR              = 503;
        /**
        * The operation is not supported for current configuration.
        * @type Number
        * @constant
        */
        this.CONNMGR_OPERATION_NOTSUPPORTED_ERR = 504;
        /**
        * The operation cannot currently be performed for the current configuration. 
        * User can try this operation after some time.
        * @type Number
        * @constant
        */
        this.CONNMGR_NO_CONFIGURATION_ERROR     = 505;
        /**
        * Connection Refused
        * @type Number
        * @constant
        */
        this.FU_CONNECTION_REFUSED_ERR        = 601;
        /**
        * Remote Host Closed
        * @type Number
        * @constant
        */
        this.FU_REMOTE_HOST_CLOSED_ERR        = 602;
        /**
        * Host Not Found
        * @type Number
        * @constant
        */
        this.FU_HOST_NOT_FOUND_ERR          = 603;
        /**
        * Time Out
        * @type Number
        * @constant
        */
        this.FU_TIME_OUT_ERR            = 604;
        /**
        * Operation Canceled
        * @type Number
        * @constant
        */
        this.FU_OPERATION_CANCELLED_ERR       = 605;
        /**
        * SSL Handhake Failed
        * @type Number
        * @constant
        */
        this.FU_SSL_HANDSHAKE_FAILED        = 606;
        /**
        * Unknown Network Error
        * @type Number
        * @constant
        */
        this.FU_UNKNOWN_NETWORK_ERR         = 699;
        /**
        * Proxy Connection Refused
        * @type Number
        * @constant
        */
        this.FU_PROXY_CONECTION_REFUSED_ERR     = 701;
        /**
        * Proxy Connection Closed
        * @type Number
        * @constant
        */
        this.FU_PROXY_CONNECTION_CLOSED_ERR     = 702;
        /**
        * Proxy Not Found
        * @type Number
        * @constant
        */
        this.FU_PROXY_NOT_FOUND_ERR         = 703;
        /**
        * Proxy time Out
        * @type Number
        * @constant
        */
        this.FU_PROXY_TIME_OUT_ERR          = 704;
        /**
        * Proxy Authentication Required
        * @type Number
        * @constant
        */
        this.FU_PROXY_AUTHENTICATION_REQUIRED_ERR   = 705;
        /**
        * Unknown Proxy Error
        * @type Number
        * @constant
        */
        this.FU_UNKNOWN_PROXY_ERR           = 799;
        /**
        * Content Access Denied
        * @type Number
        * @constant
        */
        this.FU_CONTENT_ACCESS_DENIED_ERR       = 801;
        /**
        * Content Operation Not Permitted
        * @type Number
        * @constant
        */
        this.FU_CONTENT_NOT_PERMITTED_ERR       = 802;
        /**
        * Content Not Found
        * @type Number
        * @constant
        */
        this.FU_CONTENT_NOT_FOUND_ERR         = 803;
        /**
        * Authentication Required
        * @type Number
        * @constant
        */
        this.FU_AUTHENTICATION_REQUIRED_ERR     = 804;
        /**
        * Content Resend
        * @type Number
        * @constant
        */
        this.FU_CONTENT_RESEND_ERR          = 805;
        /**
        * Unknown Content Error
        * @type Number
        * @constant
        */
        this.FU_CONTENT_UNKNOWN_ERR         = 899;
        /**
        * Protocol Unknown Error
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_UNKNOWN_ERR        = 901;
        /**
        * Protocol Invalid Operation
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_INVALID_OPERATION      = 902;
        /**
        * Protocol Failure
        * @type Number
        * @constant
        */
        this.FU_PROTOCOL_FAILURE_ERR        = 999;
    };

wrt.device.DeviceAPIError.prototype = new Error();


wrt.device.DeviceAPIError.prototype.toString = function(){
    return(this.name + " : " + this.code + " : " + this.message);
};

/**
 * @class
 * @description The Iterator object is used for storing and efficiently traversing through a list of objects. Each 
 * Platform Services 2.0 API uses an iterator specific to the service. For example, the getList method 
 * of the Calendar API returns an iterator of CalendarEntry objects, while the getList method of the 
 * Media Management API returns an iterator of MediaFileData objects.
 */
wrt.device.DeviceIterator = function (aIter){
    //internal copy
    this.__iter = aIter;
    /**
     * @function
     * @description The next method retrieves the next object in the iterator.
     * @return {Object}   Method returns next traversed object in the iterator. If all objects have been iterated, 
     * the method returns null.
     */
     this.next = function(){
        var item = this.__iter.next();
        if (item)
            return item;
        else
            return null;
    };
    /**
     * @function
     * @description The hasNext method checks whether there are uniterated objects left in the iterator.
     * @return {Boolean}   Method returns a boolean value. If there are uniterated objects left in the iterator, 
     * the method returns true. If all objects have been iterated, the method returns false.
     */
    this.hasNext = function(){
        return this.__iter.hasNext();
    };
    this.reset = function(){
        return this.__iter.reset();
    };
    /**
     * @function
     * @description The close method frees up any resources allocated to the iterator.
     */
    this.close = function(){
        return this.__iter.close();
    };
}


var asyncCallbackMap = {
    usec_count:0,
    callbackArray: new Array(),
    getTransactionId: function(){
        var KHrs=24,KMin=60,KSec=60,KMcSec=1000;
        var currentTime = new Date();
        this.usec_count++;
        /*return   ((KMcSec * KHrs * KMin * KSec * currentTime.getDate())+
        (KMcSec * KMin * KSec * currentTime.getHours())+
        (KMcSec * KSec * currentTime.getMinutes())+
        (KMcSec * currentTime.getSeconds()) +*/
        return this.usec_count;
    },
    add: function(sCb, errCb,options){
        var obj = new Object();
        obj.success_cb = sCb;
        obj.error_cb = errCb;
        obj.options = options;
        obj.transactionId = this.getTransactionId();
        this.callbackArray.push(obj);
        return obj.transactionId;
    },
    get: function(tid){
        var i = this.callbackArray.length -1;
        for (; i >= 0; i--) {
            if (this.callbackArray[i].transactionId == tid) {
                return this.callbackArray[i];
            }
        }
        return null;
    },
    remove: function(tid){
        var i = this.callbackArray.length -1;
        for (; i >= 0; i--) {
            if (this.callbackArray[i].transactionId == tid) {
                this.callbackArray.splice(i, 1);
            }
        }
    }

};

/*
* attach geolocation interface to navigator object.
*/
if(!window.navigator.geolocation){
    window.navigator.__defineGetter__("geolocation",function(){
        try{
            if(!window.wrt.device.geolocation){
                wrt.device.load('geolocation');
            }
            return window.wrt.device.geolocation;
        }catch(e){
            return window.wrt.device.geolocation;
        }
    });
}

//namespace alias
window.nokia = window.wrt;
