/*
 * Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This file is part of Qt Web Runtime.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

if ((typeof wrt.device.telephony) == 'undefined') {
/**
* A wrapper JS class for objects implementing ITelephony interface of Telephony Plugin
*
* @namespace
*/
    wrt.device.telephony = (function(){

    var DeviceException = wrt.device.DeviceException;

    var error = new DeviceException(0, 'dummy');

    /*
     * Internal
     * qt telephony interface object.
     */
      var qtTelephonyIf = wrt._services.load("wrt.device.telephony", "com.nokia.ITelephony", "1.0");	
       
    return {
        /**
        * @constant {String}
        * interfaceName.
        */
        interfaceName: "telephony",
        version: wrt.device._interfaceVersion,
        /**
         * Initiates a voice call to a number provided by user.
         * This is a synchronous API.
         *
         * @param {String}
         *          phoneNumber The phone number to which a voice call will be made.
         *
         * @return {void}
         *
         * @exception {DeviceException}  Code 1 (MISSING_ARG_ERR): An argument passed is not an expected type or is invalid. Size of input string may be platform dependent.
         *                    <ul><li>If phoneNumber is missing.</li></ul>
         *                 Code 2 (INVALID_ARG_ERR): A mandatory argument in a method is missing.<ul><li>Invalid Phone number.</li></ul>
         *
         */
        initiateVoiceCall: function(phoneNumber){

            //input validations.
            if (!phoneNumber) {
                throw new DeviceException(error.MISSING_ARG_ERR, 'mandatory parameter phoneNumber is missing');
            }
            else
                if ((typeof phoneNumber) != "string") {
                    throw new DeviceException(error.INVALID_ARG_ERR, 'phoneNumber param must be a string');
                }

            var ret = qtTelephonyIf.initiateVoiceCall(phoneNumber);
            if (ret["errorCode"] == -2 ||ret["errorCode"] == -6) {
                throw new DeviceException(error.INVALID_ARG_ERR, ret["errorMessage"]);
            }
        }
        }; //End return
    })();//End sensors
}// end if
